@@ -3,55 +3,64 @@ import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib as mpl
 import pyvista as pv
-from tqdm import tqdm
+from tqdm import tqdm, trange
 import tetgen
 from scipy import stats
 import pymeshfix
 import pickle
+import os
+from scipy.spatial import cKDTree
 
 q = 4
 resolution = 120
 
+def get_cube():
+    cu = pv.Cube(x_length=3.72,y_length=3.72,z_length=3.72).triangulate().subdivide(5)
+    cube = svcco.surface()
+    cube.set_data(cu.points,cu.point_normals)
+    cube.solve()
+    cube.build(q=q,resolution=resolution)
+    print('cube constructed')
+    return cube
 
-cu = pv.Cube(x_length=3.72,y_length=3.72,z_length=3.72).triangulate().subdivide(5)
-cube = svcco.surface()
-cube.set_data(cu.points,cu.point_normals)
-cube.solve()
-cube.build(q=q,resolution=resolution)
-print('cube constructed')
+def get_heart():
+    heart = svcco.surface()
+    heart_points = np.genfromtxt('D:\\svcco\\svcco\\implicit\\tests\\heart_points_unique.csv',delimiter=',')
+    heart_normals = np.genfromtxt('D:\\svcco\\svcco\\implicit\\tests\\heart_normals_unique.csv',delimiter=',')
+    heart.set_data(heart_points,heart_normals)
+    heart.solve()
+    heart.build(q=q,resolution=resolution,k=2,buffer=5)
+    print('heart constructed')
+    return heart
 
-heart = svcco.surface()
-heart_points = np.genfromtxt('D:\\svcco\\svcco\\implicit\\tests\\heart_points_unique.csv',delimiter=',')
-heart_normals = np.genfromtxt('D:\\svcco\\svcco\\implicit\\tests\\heart_normals_unique.csv',delimiter=',')
-heart.set_data(heart_points,heart_normals)
-heart.solve()
-heart.build(q=4,resolution=120,k=2,buffer=5)
-print('heart constructed')
+def get_disk():
+    disk = pv.Disc(inner=2.8,outer=4,r_res=20,c_res=100)
+    cyl  = disk.extrude([0,0,2],capping=True).triangulate()
+    cyl  = svcco.utils.remeshing.remesh.remesh_surface(cyl)
+    cyl  = cyl.subdivide(2)
+    cyl  = svcco.utils.remeshing.remesh.remesh_surface(cyl,hausd=0.005)
+    cyl  = cyl.compute_normals(auto_orient_normals=True,feature_angle=90)
+    cylinder = svcco.surface()
+    cylinder.set_data(cyl.points,cyl.point_normals)
+    cylinder.solve()
+    cylinder.build(q=q,resolution=resolution)
+    print('cylinder constructed')
+    return cylinder
 
-
-disk = pv.Disc(inner=2.8,outer=4,r_res=20,c_res=100)
-cyl  = disk.extrude([0,0,2],capping=True).triangulate()
-cyl  = svcco.utils.remeshing.remesh.remesh_surface(cyl)
-cyl  = cyl.subdivide(2)
-cyl  = svcco.utils.remeshing.remesh.remesh_surface(cyl,hausd=0.005)
-cyl  = cyl.compute_normals(auto_orient_normals=True,feature_angle=90)
-cylinder = svcco.surface()
-cylinder.set_data(cyl.points,cyl.point_normals)
-cylinder.solve()
-cylinder.build(q=q,resolution=resolution)
-print('cylinder constructed')
-
-left_gyrus   = "D:\\Tree\\Tree_8-0\\brain_testing\\FJ3801_BP58201_FMA72658_Left inferior frontal gyrus.obj"
-gyrus_no_scale = pv.read(left_gyrus)
-sf = (heart.volume/gyrus_no_scale.volume)**(1/3)
-gyrus_scaled = gyrus_no_scale.scale([sf,sf,sf])
-left_gyrus_scaled = "left_gyrus_scaled.vtp"
-gyrus_scaled.save(left_gyrus_scaled)
-gyrus = svcco.surface()
-gyrus.load(left_gyrus_scaled)
-gyrus.solve()
-gyrus.build(q=q,resolution=resolution,buffer=5)
-print('gyrus constructed')
+def get_gyrus():
+    left_gyrus   = "D:\\Tree\\Tree_8-0\\brain_testing\\FJ3801_BP58201_FMA72658_Left inferior frontal gyrus.obj"
+    gyrus_no_scale = pv.read(left_gyrus)
+    heart = get_heart()
+    sf = (heart.volume/gyrus_no_scale.volume)**(1/3)
+    gyrus_scaled = gyrus_no_scale.scale([sf,sf,sf])
+    left_gyrus_scaled = "left_gyrus_scaled.vtp"
+    gyrus_scaled.save(left_gyrus_scaled)
+    gyrus = svcco.surface()
+    gyrus.load(left_gyrus_scaled)
+    gyrus.solve()
+    gyrus.build(q=q,resolution=resolution,buffer=5)
+    print('gyrus constructed')
+    return gyrus
 
 ###########################################
 # Test function
@@ -95,6 +104,53 @@ def perfusion_volumes(tree,subdivisions=0,tet=None):
     return territory_id,territory_volumes/np.sum(territory_volumes),tet
 
 
+def preprocess(surf_object):
+    surf_object.cell_centers = surf_object.tet.grid.cell_centers().points
+    surf_object.cell_center_closest_point = []
+    KDTREE = cKDTree(surf_object.tet.grid.points)
+    surf_object.TET_KDTREE = KDTREE
+    return
+
+def subpoints(pt1,pt2,num):
+    t = np.linspace(0,1,num).reshape(-1,1)
+    return pt2*t+(1-t)*pt1
+
+def perfusion_simple(tree,perfusion_volume):
+    terminals = tree.data[tree.data[:,15]<0,:]
+    terminals = terminals[terminals[:,16]<0,:]
+    element_volumes = perfusion_volume.tet.grid.compute_cell_sizes().cell_data['Volume']
+    terminal_territory_volumes = np.zeros(terminals.shape[0]+1)
+    n_cells = perfusion_volume.tet.grid.n_cells
+    territory_ids = []
+    for idx in trange(n_cells):
+        pt1 = perfusion_volume.cell_centers[idx,:]
+        distances = np.linalg.norm(terminals[:,3:6] - pt1.reshape(1,-1),axis=1).flatten()
+        ordered_terminals = np.argsort(distances).flatten()
+        territory_id = None
+        perfused = False
+        for jdx in ordered_terminals:
+            pt2 = terminals[jdx,3:6]
+            line_pts = subpoints(pt1,pt2,20)
+            next_line = False
+            for kdx in range(line_pts.shape[0]):
+                if not perfusion_volume.within(line_pts[kdx,0],line_pts[kdx,1],line_pts[kdx,2],2):
+                    next_line = True
+                    break
+            if next_line:
+                perfused = False
+                continue
+            else:
+                perfused = True
+                territory_id = jdx
+        if perfused:
+            territory_ids.append(territory_id)
+            terminal_territory_volumes[territory_id] += element_volumes[idx]
+        else:
+            territory_ids.append(-1)
+            terminal_territory_volumes[-1] += element_volumes[idx]
+    territory_ids = np.array(territory_ids)
+    return territory_ids,terminal_territory_volumes/np.sum(element_volumes),perfusion_volume.tet
+
 def perfusion_geodesic(tree,perfusion_volume):
     terminals = tree.data[tree.data[:,15]<0,:]
     terminals = terminals[terminals[:,16]<0,:]
@@ -131,25 +187,33 @@ def perfusion_geodesic(tree,perfusion_volume):
     return territory_id,territory_volumes/np.sum(vol),perfusion_volume.tet
 
 
-def test(surf_object,size=1000,restarts=50):
+def test(surf_object,size=1000,restarts=50,name='default'):
     t = svcco.tree()
     t.set_boundary(surf_object)
+    perfusion_folder = os.getcwd()+os.sep+name+os.sep
     delaunay = surf_object.pv_polydata.delaunay_3d()
     convexity = surf_object.volume/delaunay.volume
     if convexity > 0.95:
         t.convex = True
     t.set_root()
     add_amount = size//restarts
+    TERS  = []
     VOLS  = []
     COUNT = []
-    _,vols,_ = perfusion_volumes(t)
+    _,vols,_ = perfusion_simple(t,surf_object)
     for i in range(restarts):
         t.n_add(add_amount)
         if i == 0:
-            _,vols,tet = perfusion_volumes(t)
+            ters,vols,tet = perfusion_simple(t,surf_object)
+            np.save(perfusion_folder + 'perfusion_volumes_num_terminals_{}.npy'.format(i*add_amount+1),vols)
+            np.save(perfusion_folder + 'territories_{}.npy'.format(i*add_amount+1),ters)
+            tet.grid.save(perfusion_folder + 'perfusion_mesh.vtu')
         else:
-            _,vols,tet = perfusion_volumes(t,tet=tet)
+            ters,vols,tet = perfusion_simple(t,surf_object)
+            np.save(perfusion_folder + 'perfusion_volumes_num_terminals_{}.npy'.format(i*add_amount+1),vols)
+            np.save(perfusion_folder + 'territories_{}.npy'.format(i*add_amount+1),ters)
         VOLS.append(vols)
+        TERS.append(ters)
         if i == 0:
             COUNT.append(add_amount)
         else:
@@ -297,7 +361,30 @@ def get_hist_data(data,hist_range=(0,1),bins=50):
         data['widths'].append(temp_width)
     return data
 
-f,a,data = results([cube,cylinder,heart,gyrus],repeat=10)
-file = open('perfusion_data.pkl','wb+')
-pickle.dump(data,file)
-file.close()
+#f,a,data = results([cube,cylinder,heart,gyrus],repeat=10)
+#file = open('perfusion_data.pkl','wb+')
+#pickle.dump(data,file)
+#file.close()
+
+"""
+cube = get_cube()
+preprocess(cube)
+results = test(cube,size=50,restarts=50,name='cube')
+print('done')
+
+heart = get_heart()
+preprocess(heart)
+results = test(heart,size=50,restarts=50,name='heart')
+print('done')
+
+disk = get_disk()
+preprocess(disk)
+results = test(disk,size=50,restarts=50,name='disk')
+print('done')
+
+gyrus = get_gyrus()
+preprocess(gyrus)
+results = test(gyrus,size=50,restarts=50,name='gyrus')
+print('done')
+
+"""
